#!/usr/bin/env python3

"""Create an empty archive in the RADAR service to reserve a DOI and a RADAR ID.

Description
-----------

This script creates an empty archive in the RADAR service in order to reserve a DOI and a RADAR ID.
Both are stored in the CodeMeta metadata file provided as input and can be later used by the script ``create_radar.py``
to populate the RADAR archive. 

Usage
-----

.. argparse::
    :module: facile_rs.prepare_radar
    :func: create_parser
    :prog: prepare_radar.py

"""

import argparse
from pathlib import Path

from .utils import settings
from .utils.metadata import CodemetaMetadata, RadarMetadata
from .utils.radar import create_radar_dataset, fetch_radar_token, prepare_radar_dataset


def create_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--radar-url', dest='radar_url',
                        help='URL of the RADAR service.')
    parser.add_argument('--radar-username', dest='radar_username',
                        help='Username for the RADAR service.')
    parser.add_argument('--radar-password', dest='radar_password',
                        help='Password for the RADAR service.')
    parser.add_argument('--radar-client-id', dest='radar_client_id',
                        help='Client ID for the RADAR service.')
    parser.add_argument('--radar-client-secret', dest='radar_client_secret',
                        help='Client secret for the RADAR service.')
    parser.add_argument('--radar-workspace-id', dest='radar_workspace_id',
                        help='Workspace ID for the RADAR service.')
    parser.add_argument('--radar-redirect-url', dest='radar_redirect_url',
                        help='Redirect URL for the OAuth workflow of the RADAR service.')
    parser.add_argument('--radar-email', dest='radar_email',
                        help='Email for the RADAR metadata.')
    parser.add_argument('--radar-backlink', dest='radar_backlink',
                        help='Backlink for the RADAR metadata.')
    parser.add_argument('--dry', action='store_true',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'RADAR_URL',
        'RADAR_CLIENT_ID',
        'RADAR_CLIENT_SECRET',
        'RADAR_REDIRECT_URL',
        'RADAR_USERNAME',
        'RADAR_PASSWORD',
        'RADAR_WORKSPACE_ID',
        'RADAR_EMAIL',
        'RADAR_BACKLINK'
    ])

    if settings.CODEMETA_LOCATION:
        codemeta = CodemetaMetadata()
        codemeta.fetch(settings.CODEMETA_LOCATION)

        name = '{name} ({version}, in preparation)'.format(**codemeta.data)
    else:
        name = 'in preparation'

    radar_metadata = RadarMetadata({'name': name}, settings.RADAR_EMAIL, settings.RADAR_BACKLINK)
    radar_dict = radar_metadata.as_dict()

    if not settings.DRY:
        # obtain oauth token
        headers = fetch_radar_token(settings.RADAR_URL, settings.RADAR_CLIENT_ID, settings.RADAR_CLIENT_SECRET,
                                    settings.RADAR_REDIRECT_URL, settings.RADAR_USERNAME, settings.RADAR_PASSWORD)

        # create radar dataset
        dataset_id = create_radar_dataset(settings.RADAR_URL, settings.RADAR_WORKSPACE_ID, headers, radar_dict)
        dataset = prepare_radar_dataset(settings.RADAR_URL, dataset_id, headers)

        doi = dataset.get('descriptiveMetadata', {}).get('identifier', {}).get('value')
        doi_url = 'https://doi.org/' + doi

        if settings.CODEMETA_LOCATION:
            codemeta.data['@id'] = doi_url
            doi_entry = {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': doi
            }
            radar_entry = {
                '@type': 'PropertyValue',
                'propertyID': 'RADAR',
                'value': dataset_id
            }
            if 'identifier' in codemeta.data and isinstance(codemeta.data['identifier'], list):
                found_doi = False
                found_radar = False
                for identifier in codemeta.data['identifier']:
                    if identifier.get('propertyID') == 'DOI':
                        identifier['value'] = doi
                        found_doi = True
                    elif identifier.get('propertyID') == 'RADAR':
                        identifier['value'] = dataset_id
                        found_radar = True
                if not found_doi:
                    codemeta.data['identifier'].append(doi_entry)
                if not found_radar:
                    codemeta.data['identifier'].append(radar_entry)
            else:
                codemeta.data['identifier'] = [doi_entry, radar_entry]

            Path(settings.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())
        else:
            print(dataset)


if __name__ == "__main__":
    main()



import json
import logging
import shutil
from pathlib import Path
from urllib.parse import urlparse

import requests
import yaml

logger = logging.getLogger(__file__)


def fetch_files(locations, path):
    """Fetch files from local locations or a URLs and save them at the given path.  
    
    :param locations: list of URL or paths to the files
    :type location: list of str
    :param path: location where the files should be saved
    :type path: str
    """
    for location in locations:
        target = path / location.split('/')[-1]

        logger.debug('location = %s, target = %s', location, target)

        if urlparse(location).scheme:
            response = requests.get(location)
            response.raise_for_status()
            with open(target, 'wb') as f:
                f.write(response.content)

        else:
            shutil.copyfile(location, target)


def fetch_dict(location):
    """Fetch data from a JSON or YAML file and return it as a dictionary.  
    
    :param location: URL or path to the file (allowed extensions: .json, .yml, .yaml)
    :type location: str
    :return: Dictionary containing file data
    :rtype: dict 
    """
    parsed_url = urlparse(location)
    if parsed_url.scheme:
        logger.debug('location = %s', location)

        response = requests.get(location)
        response.raise_for_status()

        if parsed_url.path.endswith('.json'):
            return json.loads(response.text)
        elif parsed_url.path.endswith('.yml') or parsed_url.path.endswith('.yaml'):
            return yaml.safe_load(response.text)
        else:
            raise RuntimeError('{} is not a JSON or YAML file.')
    else:
        path = Path(location).expanduser()
        with open(Path(location).expanduser()) as f:
            if path.suffix == '.json':
                return json.load(f)
            elif path.suffix in ['.yml', '.yaml']:
                return yaml.safe_load(f.read())
            else:
                raise RuntimeError('{} is not a JSON or YAML file.')


def fetch_json(location):
    """
    Fetch JSON data from the given location.

    :param location: URL to the JSON data
    :type location: str
    :return: JSON-encoded response
    """
    response = requests.get(location, headers={'Accept': 'application/json'})
    response.raise_for_status()
    return response.json()



#!/usr/bin/env python3

"""Create an archive in the RADAR service.

Description
-----------

This script creates an archive in the RADAR service and uploads the assets provided as positional arguments.
The metadata is created similar to create_datacite.

If the RADAR ID is already present in the CodeMeta file, the existing RADAR archive is updated instead.

Usage
-----

.. argparse::
    :module: facile_rs.create_radar
    :func: create_parser
    :prog: create_radar.py

"""

import argparse
import smtplib
from pathlib import Path

from .utils import settings
from .utils.http import fetch_files
from .utils.metadata import CodemetaMetadata, RadarMetadata
from .utils.radar import create_radar_dataset, fetch_radar_token, update_radar_dataset, upload_radar_assets

def create_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('assets', nargs='*', default=[],
                        help='Assets to be added to the repository.')
    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--creators-location', dest='creators_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-location', dest='contributors_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--no-sort-authors', dest='sort_authors', action='store_false',
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.set_defaults(sort_authors=True)
    parser.add_argument('--radar-path', dest='radar_path',
                        help='Path to the Radar directory, where the assets are collected before upload.')
    parser.add_argument('--radar-url', dest='radar_url',
                        help='URL of the RADAR service.')
    parser.add_argument('--radar-username', dest='radar_username',
                        help='Username for the RADAR service.')
    parser.add_argument('--radar-password', dest='radar_password',
                        help='Password for the RADAR service.')
    parser.add_argument('--radar-client-id', dest='radar_client_id',
                        help='Client ID for the RADAR service.')
    parser.add_argument('--radar-client-secret', dest='radar_client_secret',
                        help='Client secret for the RADAR service.')
    parser.add_argument('--radar-workspace-id', dest='radar_workspace_id',
                        help='Workspace ID for the RADAR service.')
    parser.add_argument('--radar-redirect-url', dest='radar_redirect_url',
                        help='Redirect URL for the OAuth workflow of the RADAR service.')
    parser.add_argument('--radar-email', dest='radar_email',
                        help='Email for the RADAR metadata.')
    parser.add_argument('--radar-backlink', dest='radar_backlink',
                        help='Backlink for the RADAR metadata.')
    parser.add_argument('--smtp-server', dest='smtp_server',
                        help='SMTP server used to inform about new relase. No mail sent if empty.')
    parser.add_argument('--notification-email', dest='notification_email',
                        help='Recipient address to inform about new relase. No mail sent if empty.')
    parser.add_argument('--dry', action='store_true',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'CODEMETA_LOCATION',
        'RADAR_PATH',
        'RADAR_URL',
        'RADAR_CLIENT_ID',
        'RADAR_CLIENT_SECRET',
        'RADAR_REDIRECT_URL',
        'RADAR_USERNAME',
        'RADAR_PASSWORD',
        'RADAR_WORKSPACE_ID',
        'RADAR_EMAIL',
        'RADAR_BACKLINK'
    ])

    # setup the bag directory
    radar_path = Path(settings.RADAR_PATH).expanduser()
    if radar_path.exists():
        parser.error(f'{radar_path} already exists.')
    radar_path.mkdir()

    # prepare radar payload
    codemeta = CodemetaMetadata()
    codemeta.fetch(settings.CODEMETA_LOCATION)
    codemeta.fetch_authors(settings.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(settings.CONTRIBUTORS_LOCATIONS)
    codemeta.compute_names()
    codemeta.remove_doubles()
    if settings.SORT_AUTHORS:
        codemeta.sort_persons()

    codemeta.data['name'] = '{name} ({version})'.format(**codemeta.data)  # override name/title to include version

    if not codemeta.data.get('copyrightHolder'):
        codemeta.data['copyrightHolder'] = [{
            'name': 'The authors'
        }]

    radar_metadata = RadarMetadata(codemeta.data, settings.RADAR_EMAIL, settings.RADAR_BACKLINK)
    radar_dict = radar_metadata.as_dict()

    # collect assets
    fetch_files(settings.ASSETS, radar_path)

    if not settings.DRY:
        # obtain oauth token
        headers = fetch_radar_token(settings.RADAR_URL, settings.RADAR_CLIENT_ID, settings.RADAR_CLIENT_SECRET,
                                    settings.RADAR_REDIRECT_URL, settings.RADAR_USERNAME, settings.RADAR_PASSWORD)

        # update or create radar dataset
        if radar_dict.get('id'):
            dataset_id = update_radar_dataset(settings.RADAR_URL, radar_dict.get('id'), headers, radar_dict)
        else:
            dataset_id = create_radar_dataset(settings.RADAR_URL, settings.RADAR_WORKSPACE_ID, headers, radar_dict)

        # upload assets
        upload_radar_assets(settings.RADAR_URL, dataset_id, headers, settings.ASSETS, radar_path)

    if settings.SMTP_SERVER and settings.NOTIFICATION_EMAIL:
        message = """\
From: {}
To: {}
Subject: {}

{}
""".format(
    settings.RADAR_EMAIL,
    settings.NOTIFICATION_EMAIL,
    "New RADAR release ready to publish",
    "A new RADAR release has been uploaded by a CI pipeline.\n\n Please visit"
    " https://radar.kit.edu/radar/de/workspace/{}.{} to publish this release.".format(
        settings.RADAR_WORKSPACE_ID,settings.RADAR_CLIENT_ID
))
        server = smtplib.SMTP(settings.SMTP_SERVER)
        server.sendmail(settings.RADAR_EMAIL, settings.NOTIFICATION_EMAIL, message)
        server.quit()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Updates the CodeMeta file with the given ``version`` and ``date``.

Useful to automatically get the version from a git tag and inject it into the repository's metadata file.
The current date is used if no date is provided.

Usage
-----

.. argparse::
   :module: facile_rs.prepare_release
   :func: create_parser
   :prog: prepare_release.py

"""

import argparse
from datetime import date
from pathlib import Path

from .utils import settings
from .utils.metadata import CodemetaMetadata

def create_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--version', dest='version',
                        help='Version of the resource')
    parser.add_argument('--date', dest='date',
                        help='Date for dateModified (format: \'%%Y-%%m-%%d\')')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')
    return parser

def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'CODEMETA_LOCATION',
        'VERSION'
    ])

    codemeta = CodemetaMetadata()
    codemeta.fetch(settings.CODEMETA_LOCATION)

    codemeta.data['version'] = settings.VERSION
    codemeta.data['dateModified'] = settings.DATE or date.today().strftime('%Y-%m-%d')

    Path(settings.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Create a BagIt bag including a DataCite XML file.

Description
-----------

This script creates a BagIt bag using the bagit-python package, but also includes a DataCite XML file
as recommended by the RDA Research Data Repository Interoperability WG.
The assets to be included in the bag are given as positional arguments.

Usage
-----

.. argparse::
    :module: facile_rs.create_bagpack
    :func: create_parser
    :prog: create_bagpack.py

"""

import argparse
from pathlib import Path

import bagit

from .utils import settings
from .utils.checksum import get_sha256, get_sha512
from .utils.http import fetch_dict, fetch_files


def create_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument('assets', nargs='*', default=[],
                        help='Assets to be added to the bag.')
    parser.add_argument('--bag-path', dest='bag_path',
                        help='Path to the Bag directory')
    parser.add_argument('--bag-info-location', dest='bag_info_locations', action='append', default=[],
                        help='Locations of the bag-info YAML/JSON files')
    parser.add_argument('--datacite-path', dest='datacite_path',
                        help='Path to the DataCite XML file')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')

    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'BAG_PATH',
        'DATACITE_PATH'
    ])

    # setup the bag
    bag_path = Path(settings.BAG_PATH).expanduser()
    if bag_path.exists():
        parser.error(f'{bag_path} already exists.')
    bag_path.mkdir()

    # collect assets
    fetch_files(settings.ASSETS, bag_path)

    # fetch bag-info
    bag_info = {}
    for location in settings.BAG_INFO_LOCATIONS:
        bag_info.update(fetch_dict(location))

    # create bag using bagit
    bag = bagit.make_bag(bag_path, bag_info)
    bag.save()

    # get datacite.xml and put it in the bag
    datacite_path = Path(settings.DATACITE_PATH).expanduser()
    datacite_xml = datacite_path.read_text()
    datacite_bag_path = bag_path / 'metadata' / 'datacite.xml'
    datacite_bag_path.parent.mkdir()
    datacite_bag_path.write_text(datacite_xml)

    with open(bag_path / 'tagmanifest-sha256.txt', 'a') as f:
        f.write(f'{get_sha256(datacite_path)} metadata/datacite.xml\n')

    with open(bag_path / 'tagmanifest-sha512.txt', 'a') as f:
        f.write(f'{get_sha512(datacite_path)} metadata/datacite.xml\n')


if __name__ == "__main__":
    main()



import pytest
import sys
from os import path

from facile_rs.create_cff import main


SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_LOCATION = path.join(METADATA_DIR, 'codemeta_test.json')
CREATORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_authors_test.json')
CONTRIBUTORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_contributors_test.json')


def test_cli(monkeypatch, tmpdir):
    output_cff = tmpdir.join('output.cff')
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--creators-location', CREATORS_LOCATIONS,
                            '--contributors-location', CONTRIBUTORS_LOCATIONS,
                            '--cff-path', str(output_cff)
                        ])
    main()
    with open(path.join(SCRIPT_DIR, 'cff_ref.cff'), 'r') as cff_ref:
        assert output_cff.read() == cff_ref.read()


def test_env(monkeypatch, tmpdir):
    output_cff = tmpdir.join('output.cff')
    monkeypatch.setenv('CODEMETA_LOCATION', CODEMETA_LOCATION)
    monkeypatch.setenv('CREATORS_LOCATIONS', CREATORS_LOCATIONS)
    monkeypatch.setenv('CONTRIBUTORS_LOCATIONS', CONTRIBUTORS_LOCATIONS)
    monkeypatch.setenv('CFF_PATH', str(output_cff))
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                        ])
    main()
    with open(path.join(SCRIPT_DIR, 'cff_ref.cff'), 'r') as cff_ref:
        assert output_cff.read() == cff_ref.read()


#!/usr/bin/env python3

"""Create a release in GitLab using the GitLab API.

Description
-----------

This script creates a release in GitLab using the GitLab API.
A tag for the release needs to be created beforehand and provided to the script.

Usage
-----

.. argparse::
   :module: facile_rs.create_release
   :func: create_parser
   :prog: create_release.py

"""

import argparse
import logging

import requests

from .utils import settings


logger = logging.getLogger(__file__)


def create_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument('assets', nargs='*', default=[],
                        help='Assets to be included in the release.')
    parser.add_argument('--release-tag', dest='release_tag',
                        help='Tag for the release.')
    parser.add_argument('--release-description', dest='release_description',
                        help='Description for the release.')
    parser.add_argument('--release-api-url', dest='release_api_url',
                        help='API URL to create the release.')
    parser.add_argument('--private-token', dest='private_token',
                        help='The PRIVATE_TOKEN to be used with the GitLab API.')
    parser.add_argument('--dry', action='store_true',
                        help='Perform a dry run, do not perfrom the final request.')
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')

    return parser


def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'RELEASE_TAG',
        'RELEASE_API_URL',
        'PRIVATE_TOKEN'
    ])

    assets = []
    for asset_location in settings.ASSETS:
        assets.append({
            'name': asset_location.split('/')[-1],
            'url': asset_location
        })

    release_json = {
        'name': settings.RELEASE_TAG,
        'tag_name': settings.RELEASE_TAG
    }

    if settings.RELEASE_DESCRIPTION:
        release_json['description'] = settings.RELEASE_DESCRIPTION.strip()

    if assets:
        release_json['assets'] = {
            'links': assets
        }

    if settings.DRY:
        print(release_json)
    else:
        logging.debug('release_json = %s', release_json)
        response = requests.post(settings.RELEASE_API_URL, headers={
            'Content-Type': 'application/json',
            'Private-Token': settings.PRIVATE_TOKEN
        }, json=release_json)
        response.raise_for_status()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Create a DataCite XML file from a CodeMeta JSON file.

Description
-----------

Create a DataCite XML file following the DataCite Metadata Schema 4.3, from one or several CodeMeta metadata files.
The metadata can be provided via a (list of) location(s) given as URL or local file path.

Usage
-----

.. argparse::
   :module: facile_rs.create_datacite
   :func: create_parser
   :prog: create_datacite.py

"""

import argparse
from pathlib import Path

from .utils import settings
from .utils.metadata import CodemetaMetadata, DataciteMetadata

def create_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument('--codemeta-location', dest='codemeta_location',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--creators-location', dest='creators_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-location', dest='contributors_locations', action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--datacite-path', dest='datacite_path',
                        help='Path to the DataCite XML output file')
    parser.add_argument('--no-sort-authors', dest='sort_authors', action='store_false',
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.set_defaults(sort_authors=True)
    parser.add_argument('--log-level', dest='log_level',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='log_file',
                        help='Path to the log file')

    return parser

def main():
    parser = create_parser()

    settings.setup(parser, validate=[
        'CODEMETA_LOCATION'
    ])

    codemeta = CodemetaMetadata()
    codemeta.fetch(settings.CODEMETA_LOCATION)
    codemeta.fetch_authors(settings.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(settings.CONTRIBUTORS_LOCATIONS)
    codemeta.compute_names()
    codemeta.remove_doubles()
    if settings.SORT_AUTHORS:
        codemeta.sort_persons()

    datacite_metadata = DataciteMetadata(codemeta.data)
    datacite_xml = datacite_metadata.to_xml()

    if settings.DATACITE_PATH:
        Path(settings.DATACITE_PATH).expanduser().write_text(datacite_xml)
    else:
        print(datacite_xml)


if __name__ == "__main__":
    main()



import json
import logging

from ..http import fetch_dict

logger = logging.getLogger(__file__)


class CodemetaMetadata:

    """A class for storing and manipulating metadata in the CodeMeta format"""

    def __init__(self):
        """Initialize metadata set as an empty dictionary."""
        self.data = {}

    def fetch(self, location):
        """Update metadata set with data fetched from a CodeMeta file
        
        :param location: URL or path to the CodeMeta JSON file
        :type location: str
        """
        if location:
            self.data.update(fetch_dict(location))

    def fetch_authors(self, locations):
        """Fetch authors from CodeMeta files and update metadata set

        :param locations: list of URL or paths to CodeMeta files
        :param locations: list
        """
        if locations:
            if 'author' not in self.data:
                self.data['author'] = []
            # Case when there is a unique author not contained in a list
            elif isinstance(self.data['author'], dict):
                self.data['author'] = [self.data['author']]
            for location in locations:
                self.data['author'] += fetch_dict(location).get('author', [])

    def fetch_contributors(self, locations):
        """Fetch contributors from CodeMeta files and update metadata set

        :param locations: list of URL or paths to CodeMeta files
        :param locations: list
        """
        if locations:
            if 'contributor' not in self.data:
                self.data['contributor'] = []
            # Case when there is a unique contributor not contained in a list
            elif isinstance(self.data['contributor'], dict):
                self.data['contributor'] = [self.data['contributor']]
            for location in locations:
                self.data['contributor'] += fetch_dict(location).get('contributor', [])

    def compute_names(self):
        """Add full name of authors and contributors in metadata set from given name and family name,
        under the dictionary key 'name'.
        """
        for key in ['author', 'contributor']:
            if key in self.data:
                for thing in self.data[key]:
                    if 'name' not in thing and ('givenName' in thing and 'familyName' in thing):
                        thing['name'] = '{} {}'.format(thing['givenName'], thing['familyName'])

    def remove_doubles(self):
        """Remove duplicates in authors and contributors lists, comparing names (key: name)
        and ids (key: @id).
        """
        for key in ['author', 'contributor']:
            if key in self.data:
                ids = set()
                names = set()
                things = []
                for thing in self.data[key]:
                    thing_id = thing.get('@id')
                    thing_name = thing.get('name')
                    if thing_id in ids or thing_name in names:
                        pass
                    else:
                        things.append(thing)
                        if thing_id is not None:
                            ids.add(thing_id)
                        if thing_name is not None:
                            names.add(thing_name)
                self.data[key] = things

    def sort_persons(self):
        """Sort authors and contributors alphabetically based on family name."""
        def get_key(item):
            key = item.get('familyName', item.get('name', ''))

            if item.get('@type') == 'Organization':
                # put organzations first unless they have an additionalType
                if item.get('additionalType') is None:
                    key = '!' + key
                else:
                    key = '~' + key

            return key

        if 'author' in self.data:
            self.data['author'] = sorted(self.data['author'], key=get_key)
        if 'contributor' in self.data:
            self.data['contributor'] = sorted(self.data['contributor'], key=get_key)

    def to_json(self):
        """Dump metadata set as JSON-formatted string.

        :return: metadata set as JSON-formatted string
        :rtype: str
        """
        return json.dumps(self.data, indent=2, ensure_ascii=False)
